#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    Dice,
    RevenueWithdrawalState,
    RevenueWithdrawalStateFailed,
    RevenueWithdrawalStatePending,
    RevenueWithdrawalStateSucceeded,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import RevenueWithdrawalStateType
from tests.auxil.slots import mro_slots


@pytest.fixture
def revenue_withdrawal_state():
    return RevenueWithdrawalState(RevenueWithdrawalStateTestBase.state)


@pytest.fixture
def revenue_withdrawal_state_pending():
    return RevenueWithdrawalStatePending()


@pytest.fixture
def revenue_withdrawal_state_succeeded():
    return RevenueWithdrawalStateSucceeded(
        date=RevenueWithdrawalStateTestBase.date, url=RevenueWithdrawalStateTestBase.url
    )


@pytest.fixture
def revenue_withdrawal_state_failed():
    return RevenueWithdrawalStateFailed()


class RevenueWithdrawalStateTestBase:
    state = "failed"
    date = dtm.datetime(2024, 1, 1, 0, 0, 0, 0, tzinfo=UTC)
    url = "url"


class TestRevenueWithdrawalStateWithoutRequest(RevenueWithdrawalStateTestBase):
    def test_slot_behaviour(self, revenue_withdrawal_state):
        inst = revenue_withdrawal_state
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_type_enum_conversion(self):
        assert type(RevenueWithdrawalState("failed").type) is RevenueWithdrawalStateType
        assert RevenueWithdrawalState("unknown").type == "unknown"

    def test_de_json(self, offline_bot):
        json_dict = {"type": "unknown"}
        rws = RevenueWithdrawalState.de_json(json_dict, offline_bot)
        assert rws.api_kwargs == {}
        assert rws.type == "unknown"

    @pytest.mark.parametrize(
        ("state", "subclass"),
        [
            ("pending", RevenueWithdrawalStatePending),
            ("succeeded", RevenueWithdrawalStateSucceeded),
            ("failed", RevenueWithdrawalStateFailed),
        ],
    )
    def test_de_json_subclass(self, offline_bot, state, subclass):
        json_dict = {"type": state, "date": to_timestamp(self.date), "url": self.url}
        rws = RevenueWithdrawalState.de_json(json_dict, offline_bot)

        assert type(rws) is subclass
        assert set(rws.api_kwargs.keys()) == {"date", "url"} - set(subclass.__slots__)
        assert rws.type == state

    def test_to_dict(self, revenue_withdrawal_state):
        json_dict = revenue_withdrawal_state.to_dict()
        assert json_dict == {"type": self.state}

    def test_equality(self, revenue_withdrawal_state):
        a = revenue_withdrawal_state
        b = RevenueWithdrawalState(self.state)
        c = RevenueWithdrawalState("pending")
        d = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


class TestRevenueWithdrawalStatePendingWithoutRequest(RevenueWithdrawalStateTestBase):
    def test_slot_behaviour(self, revenue_withdrawal_state_pending):
        inst = revenue_withdrawal_state_pending
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {}
        rws = RevenueWithdrawalStatePending.de_json(json_dict, offline_bot)
        assert rws.api_kwargs == {}
        assert rws.type == "pending"

    def test_to_dict(self, revenue_withdrawal_state_pending):
        json_dict = revenue_withdrawal_state_pending.to_dict()
        assert json_dict == {"type": "pending"}

    def test_equality(self, revenue_withdrawal_state_pending):
        a = revenue_withdrawal_state_pending
        b = RevenueWithdrawalStatePending()
        c = RevenueWithdrawalState("unknown")
        d = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


class TestRevenueWithdrawalStateSucceededWithoutRequest(RevenueWithdrawalStateTestBase):
    state = RevenueWithdrawalStateType.SUCCEEDED

    def test_slot_behaviour(self, revenue_withdrawal_state_succeeded):
        inst = revenue_withdrawal_state_succeeded
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"date": to_timestamp(self.date), "url": self.url}
        rws = RevenueWithdrawalStateSucceeded.de_json(json_dict, offline_bot)
        assert rws.api_kwargs == {}
        assert rws.type == "succeeded"
        assert rws.date == self.date
        assert rws.url == self.url

    def test_to_dict(self, revenue_withdrawal_state_succeeded):
        json_dict = revenue_withdrawal_state_succeeded.to_dict()
        assert json_dict["type"] == "succeeded"
        assert json_dict["date"] == to_timestamp(self.date)
        assert json_dict["url"] == self.url

    def test_equality(self, revenue_withdrawal_state_succeeded):
        a = revenue_withdrawal_state_succeeded
        b = RevenueWithdrawalStateSucceeded(date=self.date, url=self.url)
        c = RevenueWithdrawalStateSucceeded(date=self.date, url="other_url")
        d = RevenueWithdrawalStateSucceeded(
            date=dtm.datetime(1900, 1, 1, 0, 0, 0, 0, tzinfo=UTC), url=self.url
        )
        e = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


class TestRevenueWithdrawalStateFailedWithoutRequest(RevenueWithdrawalStateTestBase):
    state = RevenueWithdrawalStateType.FAILED

    def test_slot_behaviour(self, revenue_withdrawal_state_failed):
        inst = revenue_withdrawal_state_failed
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {}
        rws = RevenueWithdrawalStateFailed.de_json(json_dict, offline_bot)
        assert rws.api_kwargs == {}
        assert rws.type == "failed"

    def test_to_dict(self, revenue_withdrawal_state_failed):
        json_dict = revenue_withdrawal_state_failed.to_dict()
        assert json_dict == {"type": "failed"}

    def test_equality(self, revenue_withdrawal_state_failed):
        a = revenue_withdrawal_state_failed
        b = RevenueWithdrawalStateFailed()
        c = RevenueWithdrawalState("unknown")
        d = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import Location, PreparedInlineMessage
from telegram._utils.datetime import UTC, to_timestamp
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def prepared_inline_message():
    return PreparedInlineMessage(
        PreparedInlineMessageTestBase.id,
        PreparedInlineMessageTestBase.expiration_date,
    )


class PreparedInlineMessageTestBase:
    id = "some_uid"
    expiration_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


class TestPreparedInlineMessageWithoutRequest(PreparedInlineMessageTestBase):
    def test_slot_behaviour(self, prepared_inline_message):
        inst = prepared_inline_message
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, prepared_inline_message):
        assert prepared_inline_message.id == self.id
        assert prepared_inline_message.expiration_date == self.expiration_date

    def test_de_json(self, prepared_inline_message):
        json_dict = {
            "id": self.id,
            "expiration_date": to_timestamp(self.expiration_date),
        }
        new_prepared_inline_message = PreparedInlineMessage.de_json(json_dict, None)

        assert isinstance(new_prepared_inline_message, PreparedInlineMessage)
        assert new_prepared_inline_message.id == prepared_inline_message.id
        assert (
            new_prepared_inline_message.expiration_date == prepared_inline_message.expiration_date
        )

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot):
        json_dict = {
            "id": "some_uid",
            "expiration_date": to_timestamp(self.expiration_date),
        }
        pim = PreparedInlineMessage.de_json(json_dict, offline_bot)
        pim_raw = PreparedInlineMessage.de_json(json_dict, raw_bot)
        pim_tz = PreparedInlineMessage.de_json(json_dict, tz_bot)

        # comparing utcoffset because comparing tzinfo objects is not reliable
        offset = pim_tz.expiration_date.utcoffset()
        offset_tz = tz_bot.defaults.tzinfo.utcoffset(pim_tz.expiration_date.replace(tzinfo=None))

        assert pim.expiration_date.tzinfo == UTC
        assert pim_raw.expiration_date.tzinfo == UTC
        assert offset_tz == offset

    def test_to_dict(self, prepared_inline_message):
        prepared_inline_message_dict = prepared_inline_message.to_dict()

        assert isinstance(prepared_inline_message_dict, dict)
        assert prepared_inline_message_dict["id"] == prepared_inline_message.id
        assert prepared_inline_message_dict["expiration_date"] == to_timestamp(
            self.expiration_date
        )

    def test_equality(self, prepared_inline_message):
        a = prepared_inline_message
        b = PreparedInlineMessage(self.id, self.expiration_date)
        c = PreparedInlineMessage(self.id, self.expiration_date + dtm.timedelta(seconds=1))
        d = PreparedInlineMessage("other_uid", self.expiration_date)
        e = Location(123, 456)

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram InlineKeyboardButton."""

from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._copytextbutton import CopyTextButton
from telegram._games.callbackgame import CallbackGame
from telegram._loginurl import LoginUrl
from telegram._switchinlinequerychosenchat import SwitchInlineQueryChosenChat
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict
from telegram._webappinfo import WebAppInfo

if TYPE_CHECKING:
    from telegram import Bot


class InlineKeyboardButton(TelegramObject):
    """This object represents one button of an inline keyboard.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`text`, :attr:`url`, :attr:`login_url`, :attr:`callback_data`,
    :attr:`switch_inline_query`, :attr:`switch_inline_query_current_chat`, :attr:`callback_game`,
    :attr:`web_app`, :attr:`pay`, :attr:`style` and :attr:`icon_custom_emoji_id` are equal.

    Note:
        * Exactly one of the fields other than :attr:`text`, :attr:`icon_custom_emoji_id`, and
          :attr:`style` must be used to specify type of the button.
        * Mind that :attr:`callback_game` is not
          working as expected. Putting a game short name in it might, but is not guaranteed to
          work.
        * If your bot allows for arbitrary callback data, in keyboards returned in a response
          from telegram, :attr:`callback_data` may be an instance of
          :class:`telegram.ext.InvalidCallbackData`. This will be the case, if the data
          associated with the button was already deleted.

          .. versionadded:: 13.6
        * Since Bot API 5.5, it's now allowed to mention users by their ID in inline keyboards.
          This will only work in Telegram versions released after December 7, 2021.
          Older clients will display *unsupported message*.
        * :attr:`style` option will only work in Telegram versions released after February 9, 2026.
          Older clients will display buttons without styling.

    Warning:
        * If your bot allows your arbitrary callback data, buttons whose callback data is a
          non-hashable object will become unhashable. Trying to evaluate ``hash(button)`` will
          result in a :class:`TypeError`.

          .. versionchanged:: 13.6

        * After Bot API 6.1, only ``HTTPS`` links will be allowed in :paramref:`login_url`.

    Examples:
        * :any:`Inline Keyboard 1 <examples.inlinekeyboard>`
        * :any:`Inline Keyboard 2 <examples.inlinekeyboard2>`

    .. seealso:: :class:`telegram.InlineKeyboardMarkup`

    .. versionchanged:: 20.0
       :attr:`web_app` is considered as well when comparing objects of this type in terms of
       equality.

    .. versionchanged:: 22.7
       :attr:`style` and :attr:`icon_custom_emoji_id` are considered as well when
       comparing objects of this type in terms of equality.

    Args:
        text (:obj:`str`): Label text on the button.
        url (:obj:`str`, optional): HTTP or tg:// url to be opened when the button is pressed.
            Links ``tg://user?id=<user_id>`` can be used to mention a user by
            their ID without using a username, if this is allowed by their privacy settings.

            .. versionchanged:: 13.9
               You can now mention a user using ``tg://user?id=<user_id>``.
        login_url (:class:`telegram.LoginUrl`, optional): An ``HTTPS`` URL used to automatically
            authorize the user. Can be used as a replacement for the Telegram Login Widget.

            Caution:
                Only ``HTTPS`` links are allowed after Bot API 6.1.
        callback_data (:obj:`str` | :obj:`object`, optional): Data to be sent in a callback query
            to the bot when the button is pressed, UTF-8
            :tg-const:`telegram.InlineKeyboardButton.MIN_CALLBACK_DATA`-
            :tg-const:`telegram.InlineKeyboardButton.MAX_CALLBACK_DATA` bytes.
            If the bot instance allows arbitrary callback data, anything can be passed.

            Tip:
                The value entered here will be available in :attr:`telegram.CallbackQuery.data`.

            .. seealso:: :wiki:`Arbitrary callback_data <Arbitrary-callback_data>`

        web_app (:class:`telegram.WebAppInfo`, optional): Description of the `Web App
            <https://core.telegram.org/bots/webapps>`_  that will be launched when the user presses
            the button. The Web App will be able to send an arbitrary message on behalf of the user
            using the method :meth:`~telegram.Bot.answer_web_app_query`. Available only in
            private chats between a user and the bot. Not supported for messages sent on behalf of
            a Telegram Business account.

            .. versionadded:: 20.0
        switch_inline_query (:obj:`str`, optional): If set, pressing the button will prompt the
            user to select one of their chats, open that chat and insert the bot's username and the
            specified inline query in the input field. May be empty, in which case just the bot's
            username will be inserted. Not supported for messages sent on behalf of a Telegram
            Business account.

            Tip:
                This is similar to the parameter :paramref:`switch_inline_query_chosen_chat`,
                but gives no control over which chats can be selected.
        switch_inline_query_current_chat (:obj:`str`, optional): If set, pressing the button will
            insert the bot's username and the specified inline query in the current chat's input
            field. May be empty, in which case only the bot's username will be inserted.

            This offers a quick way for the user to open your bot in inline mode in the same chat
            - good for selecting something from multiple options. Not supported in channels and for
            messages sent on behalf of a Telegram Business account.
        copy_text (:class:`telegram.CopyTextButton`, optional): Description of the button that
            copies the specified text to the clipboard.

            .. versionadded:: 21.7
        callback_game (:class:`telegram.CallbackGame`, optional): Description of the game that will
            be launched when the user presses the button

            Note:
                This type of button **must** always be the first button in the first row.
        pay (:obj:`bool`, optional): Specify :obj:`True`, to send a Pay button.
            Substrings ``“⭐️”`` and ``“XTR”`` in the buttons's text will be replaced with a
            Telegram Star icon.

            Note:
                This type of button **must** always be the first button in the first row and can
                only be used in invoice messages.
        style (:obj:`str`, optional): Style of the button. Must be one of
            :tg-const:`telegram.constants.KeyboardButtonStyle.PRIMARY` (blue),
            :tg-const:`telegram.constants.KeyboardButtonStyle.SUCCESS` (green), and
            :tg-const:`telegram.constants.KeyboardButtonStyle.DANGER` (red).
            Color name aliases :tg-const:`telegram.constants.KeyboardButtonStyle.BLUE`,
            :tg-const:`telegram.constants.KeyboardButtonStyle.GREEN`, and
            :tg-const:`telegram.constants.KeyboardButtonStyle.RED` are also available.
            If omitted, then an app-specific style is used.

            .. versionadded:: 22.7
        switch_inline_query_chosen_chat (:class:`telegram.SwitchInlineQueryChosenChat`, optional):
            If set, pressing the button will prompt the user to select one of their chats of the
            specified type, open that chat and insert the bot's username and the specified inline
            query in the input field. Not supported for messages sent on behalf of a Telegram
            Business account.

            .. versionadded:: 20.3

            Tip:
                This is similar to :paramref:`switch_inline_query`, but gives more control on
                which chats can be selected.

            Caution:
                The PTB team has discovered that this field works correctly only if your Telegram
                client is released after April 20th 2023.
        icon_custom_emoji_id (:obj:`str`, optional): Unique identifier of the custom emoji shown
            before the text of the button. Can only be used by bots that purchased additional
            usernames on `Fragment <https://fragment.com/>`_ or in the messages directly sent by
            the bot to private, group and supergroup chats if the owner of the bot has a Telegram
            Premium subscription.

            .. versionadded:: 22.7

    Attributes:
        text (:obj:`str`): Label text on the button.
        url (:obj:`str`): Optional. HTTP or tg:// url to be opened when the button is pressed.
            Links ``tg://user?id=<user_id>`` can be used to mention a user by
            their ID without using a username, if this is allowed by their privacy settings.

            .. versionchanged:: 13.9
               You can now mention a user using ``tg://user?id=<user_id>``.
        login_url (:class:`telegram.LoginUrl`): Optional. An ``HTTPS`` URL used to automatically
            authorize the user. Can be used as a replacement for the Telegram Login Widget.

            Caution:
                Only ``HTTPS`` links are allowed after Bot API 6.1.
        callback_data (:obj:`str` | :obj:`object`): Optional. Data to be sent in a callback query
            to the bot when the button is pressed, UTF-8
            :tg-const:`telegram.InlineKeyboardButton.MIN_CALLBACK_DATA`-
            :tg-const:`telegram.InlineKeyboardButton.MAX_CALLBACK_DATA` bytes.
        web_app (:class:`telegram.WebAppInfo`): Optional. Description of the `Web App
            <https://core.telegram.org/bots/webapps>`_  that will be launched when the user presses
            the button. The Web App will be able to send an arbitrary message on behalf of the user
            using the method :meth:`~telegram.Bot.answer_web_app_query`. Available only in
            private chats between a user and the bot. Not supported for messages sent on behalf of
            a Telegram Business account.

            .. versionadded:: 20.0
        switch_inline_query (:obj:`str`): Optional. If set, pressing the button will prompt the
            user to select one of their chats, open that chat and insert the bot's username and the
            specified inline query in the input field. May be empty, in which case just the bot's
            username will be inserted. Not supported for messages sent on behalf of a Telegram
            Business account.

            Tip:
                This is similar to the parameter :paramref:`switch_inline_query_chosen_chat`,
                but gives no control over which chats can be selected.
        switch_inline_query_current_chat (:obj:`str`): Optional. If set, pressing the button will
            insert the bot's username and the specified inline query in the current chat's input
            field. May be empty, in which case only the bot's username will be inserted.

            This offers a quick way for the user to open your bot in inline mode in the same chat
            - good for selecting something from multiple options. Not supported in channels and for
            messages sent on behalf of a Telegram Business account.
        copy_text (:class:`telegram.CopyTextButton`): Optional. Description of the button that
            copies the specified text to the clipboard.

            .. versionadded:: 21.7
        style (:obj:`str`): Optional. Style of the button. Must be one of
            :tg-const:`telegram.constants.KeyboardButtonStyle.PRIMARY` (blue),
            :tg-const:`telegram.constants.KeyboardButtonStyle.SUCCESS` (green), and
            :tg-const:`telegram.constants.KeyboardButtonStyle.DANGER` (red).
            Color name aliases :tg-const:`telegram.constants.KeyboardButtonStyle.BLUE`,
            :tg-const:`telegram.constants.KeyboardButtonStyle.GREEN`, and
            :tg-const:`telegram.constants.KeyboardButtonStyle.RED` are also available.
            If omitted, then an app-specific style is used.

            .. versionadded:: 22.7
        callback_game (:class:`telegram.CallbackGame`): Optional. Description of the game that will
            be launched when the user presses the button.

            Note:
                This type of button **must** always be the first button in the first row.
        pay (:obj:`bool`): Optional. Specify :obj:`True`, to send a Pay button.
            Substrings ``“⭐️”`` and ``“XTR”`` in the buttons's text will be replaced with a
            Telegram Star icon.

            Note:
                This type of button **must** always be the first button in the first row and can
                only be used in invoice messages.
        switch_inline_query_chosen_chat (:class:`telegram.SwitchInlineQueryChosenChat`): Optional.
            If set, pressing the button will prompt the user to select one of their chats of the
            specified type, open that chat and insert the bot's username and the specified inline
            query in the input field. Not supported for messages sent on behalf of a Telegram
            Business account.

            .. versionadded:: 20.3

            Tip:
                This is similar to :attr:`switch_inline_query`, but gives more control on
                which chats can be selected.

            Caution:
                The PTB team has discovered that this field works correctly only if your Telegram
                client is released after April 20th 2023.
        icon_custom_emoji_id (:obj:`str`): Optional. Unique identifier of the custom emoji shown
            before the text of the button. Can only be used by bots that purchased additional
            usernames on `Fragment <https://fragment.com/>`_ or in the messages directly sent by
            the bot to private, group and supergroup chats if the owner of the bot has a Telegram
            Premium subscription.

            .. versionadded:: 22.7
    """

    __slots__ = (
        "callback_data",
        "callback_game",
        "copy_text",
        "icon_custom_emoji_id",
        "login_url",
        "pay",
        "style",
        "switch_inline_query",
        "switch_inline_query_chosen_chat",
        "switch_inline_query_current_chat",
        "text",
        "url",
        "web_app",
    )

    def __init__(
        self,
        text: str,
        url: str | None = None,
        callback_data: str | object | None = None,
        switch_inline_query: str | None = None,
        switch_inline_query_current_chat: str | None = None,
        callback_game: CallbackGame | None = None,
        pay: bool | None = None,
        login_url: LoginUrl | None = None,
        web_app: WebAppInfo | None = None,
        switch_inline_query_chosen_chat: SwitchInlineQueryChosenChat | None = None,
        copy_text: CopyTextButton | None = None,
        style: str | None = None,
        icon_custom_emoji_id: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.text: str = text

        # Optionals
        self.url: str | None = url
        self.login_url: LoginUrl | None = login_url
        self.callback_data: str | object | None = callback_data
        self.switch_inline_query: str | None = switch_inline_query
        self.switch_inline_query_current_chat: str | None = switch_inline_query_current_chat
        self.callback_game: CallbackGame | None = callback_game
        self.pay: bool | None = pay
        self.web_app: WebAppInfo | None = web_app
        self.switch_inline_query_chosen_chat: SwitchInlineQueryChosenChat | None = (
            switch_inline_query_chosen_chat
        )
        self.copy_text: CopyTextButton | None = copy_text
        self.style: str | None = style
        self.icon_custom_emoji_id: str | None = icon_custom_emoji_id
        self._id_attrs = ()
        self._set_id_attrs()

        self._freeze()

    def _set_id_attrs(self) -> None:
        self._id_attrs = (
            self.text,
            self.url,
            self.login_url,
            self.callback_data,
            self.web_app,
            self.switch_inline_query,
            self.switch_inline_query_current_chat,
            self.callback_game,
            self.pay,
            self.style,
            self.icon_custom_emoji_id,
        )

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "InlineKeyboardButton":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["login_url"] = de_json_optional(data.get("login_url"), LoginUrl, bot)
        data["web_app"] = de_json_optional(data.get("web_app"), WebAppInfo, bot)
        data["callback_game"] = de_json_optional(data.get("callback_game"), CallbackGame, bot)
        data["switch_inline_query_chosen_chat"] = de_json_optional(
            data.get("switch_inline_query_chosen_chat"), SwitchInlineQueryChosenChat, bot
        )
        data["copy_text"] = de_json_optional(data.get("copy_text"), CopyTextButton, bot)

        return super().de_json(data=data, bot=bot)

    def update_callback_data(self, callback_data: str | object) -> None:
        """
        Sets :attr:`callback_data` to the passed object. Intended to be used by
        :class:`telegram.ext.CallbackDataCache`.

        .. versionadded:: 13.6

        Args:
            callback_data (:class:`object`): The new callback data.
        """
        with self._unfrozen():
            self.callback_data = callback_data
            self._set_id_attrs()

    MIN_CALLBACK_DATA: Final[int] = constants.InlineKeyboardButtonLimit.MIN_CALLBACK_DATA
    """:const:`telegram.constants.InlineKeyboardButtonLimit.MIN_CALLBACK_DATA`

    .. versionadded:: 20.0
    """
    MAX_CALLBACK_DATA: Final[int] = constants.InlineKeyboardButtonLimit.MAX_CALLBACK_DATA
    """:const:`telegram.constants.InlineKeyboardButtonLimit.MAX_CALLBACK_DATA`

    .. versionadded:: 20.0
    """



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import asyncio
import datetime as dtm
import os
from pathlib import Path

import pytest

from telegram import Animation, Bot, InputFile, MessageEntity, PhotoSize, ReplyParameters, Voice
from telegram.constants import ParseMode
from telegram.error import BadRequest, TelegramError
from telegram.helpers import escape_markdown
from telegram.request import RequestData
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.build_messages import make_message
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


class AnimationTestBase:
    animation_file_id = "CgADAQADngIAAuyVeEez0xRovKi9VAI"
    animation_file_unique_id = "adc3145fd2e84d95b64d68eaa22aa33e"
    width = 320
    height = 180
    duration = dtm.timedelta(seconds=1)
    # animation_file_url = 'https://python-telegram-bot.org/static/testfiles/game.gif'
    # Shortened link, the above one is cached with the wrong duration.
    animation_file_url = "http://bit.ly/2L18jua"
    file_name = "game.gif.webm"
    mime_type = "video/mp4"
    file_size = 5859
    caption = "Test *animation*"


class TestAnimationWithoutRequest(AnimationTestBase):
    def test_slot_behaviour(self, animation):
        for attr in animation.__slots__:
            assert getattr(animation, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(animation)) == len(set(mro_slots(animation))), "duplicate slot"

    def test_creation(self, animation):
        assert isinstance(animation, Animation)
        assert isinstance(animation.file_id, str)
        assert isinstance(animation.file_unique_id, str)
        assert animation.file_id
        assert animation.file_unique_id

    def test_expected_values(self, animation):
        assert animation.mime_type == self.mime_type
        assert animation.file_name.startswith("game.gif") == self.file_name.startswith("game.gif")
        assert isinstance(animation.thumbnail, PhotoSize)

    def test_de_json(self, offline_bot, animation):
        json_dict = {
            "file_id": self.animation_file_id,
            "file_unique_id": self.animation_file_unique_id,
            "width": self.width,
            "height": self.height,
            "duration": self.duration.total_seconds(),
            "thumbnail": animation.thumbnail.to_dict(),
            "file_name": self.file_name,
            "mime_type": self.mime_type,
            "file_size": self.file_size,
        }
        animation = Animation.de_json(json_dict, offline_bot)
        assert animation.api_kwargs == {}
        assert animation.file_id == self.animation_file_id
        assert animation.file_unique_id == self.animation_file_unique_id
        assert animation.file_name == self.file_name
        assert animation.mime_type == self.mime_type
        assert animation.file_size == self.file_size
        assert animation._duration == self.duration

    def test_to_dict(self, animation):
        animation_dict = animation.to_dict()

        assert isinstance(animation_dict, dict)
        assert animation_dict["file_id"] == animation.file_id
        assert animation_dict["file_unique_id"] == animation.file_unique_id
        assert animation_dict["width"] == animation.width
        assert animation_dict["height"] == animation.height
        assert animation_dict["duration"] == int(self.duration.total_seconds())
        assert isinstance(animation_dict["duration"], int)
        assert animation_dict["thumbnail"] == animation.thumbnail.to_dict()
        assert animation_dict["file_name"] == animation.file_name
        assert animation_dict["mime_type"] == animation.mime_type
        assert animation_dict["file_size"] == animation.file_size

    def test_time_period_properties(self, PTB_TIMEDELTA, animation):
        if PTB_TIMEDELTA:
            assert animation.duration == self.duration
            assert isinstance(animation.duration, dtm.timedelta)
        else:
            assert animation.duration == int(self.duration.total_seconds())
            assert isinstance(animation.duration, int)

    def test_time_period_int_deprecated(self, recwarn, PTB_TIMEDELTA, animation):
        animation.duration

        if PTB_TIMEDELTA:
            assert len(recwarn) == 0
        else:
            assert len(recwarn) == 1
            assert "`duration` will be of type `datetime.timedelta`" in str(recwarn[0].message)
            assert recwarn[0].category is PTBDeprecationWarning

    def test_equality(self):
        a = Animation(
            self.animation_file_id,
            self.animation_file_unique_id,
            self.height,
            self.width,
            self.duration,
        )
        b = Animation("", self.animation_file_unique_id, self.height, self.width, self.duration)
        d = Animation("", "", 0, 0, 0)
        e = Voice(self.animation_file_id, self.animation_file_unique_id, 0)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_send_animation_custom_filename(
        self, offline_bot, chat_id, animation_file, monkeypatch
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            return next(iter(request_data.multipart_data.values()))[0] == "custom_filename"

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        assert await offline_bot.send_animation(
            chat_id, animation_file, filename="custom_filename"
        )

    @pytest.mark.parametrize("local_mode", [True, False])
    async def test_send_animation_local_files(
        self, monkeypatch, offline_bot, chat_id, local_mode, dummy_message_dict
    ):
        try:
            offline_bot._local_mode = local_mode
            # For just test that the correct paths are passed as we have no local Bot API set up
            test_flag = False
            file = data_file("telegram.jpg")
            expected = file.as_uri()

            async def make_assertion(_, data, *args, **kwargs):
                nonlocal test_flag
                if local_mode:
                    test_flag = (
                        data.get("animation") == expected and data.get("thumbnail") == expected
                    )
                else:
                    test_flag = isinstance(data.get("animation"), InputFile) and isinstance(
                        data.get("thumbnail"), InputFile
                    )
                return dummy_message_dict

            monkeypatch.setattr(offline_bot, "_post", make_assertion)
            await offline_bot.send_animation(chat_id, file, thumbnail=file)
            assert test_flag
        finally:
            offline_bot._local_mode = False

    async def test_send_with_animation(self, monkeypatch, offline_bot, chat_id, animation):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            return request_data.json_parameters["animation"] == animation.file_id

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        assert await offline_bot.send_animation(animation=animation, chat_id=chat_id)

    async def test_get_file_instance_method(self, monkeypatch, animation):
        async def make_assertion(*_, **kwargs):
            return kwargs["file_id"] == animation.file_id

        assert check_shortcut_signature(Animation.get_file, Bot.get_file, ["file_id"], [])
        assert await check_shortcut_call(animation.get_file, animation.get_bot(), "get_file")
        assert await check_defaults_handling(animation.get_file, animation.get_bot())

        monkeypatch.setattr(animation.get_bot(), "get_file", make_assertion)
        assert await animation.get_file()

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"parse_mode": ParseMode.HTML}, None),
            ({"parse_mode": ParseMode.HTML}, ParseMode.MARKDOWN_V2),
            ({"parse_mode": None}, ParseMode.MARKDOWN_V2),
        ],
        indirect=["default_bot"],
    )
    async def test_send_animation_default_quote_parse_mode(
        self, default_bot, chat_id, animation, custom, monkeypatch
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            assert request_data.parameters["reply_parameters"].get("quote_parse_mode") == (
                custom or default_bot.defaults.quote_parse_mode
            )
            return make_message("dummy reply").to_dict()

        kwargs = {"message_id": 1}
        if custom is not None:
            kwargs["quote_parse_mode"] = custom

        monkeypatch.setattr(default_bot.request, "post", make_assertion)
        await default_bot.send_animation(
            chat_id, animation, reply_parameters=ReplyParameters(**kwargs)
        )


class TestAnimationWithRequest(AnimationTestBase):
    @pytest.mark.parametrize("duration", [1, dtm.timedelta(seconds=1)])
    async def test_send_all_args(
        self, bot, chat_id, animation_file, animation, thumb_file, duration
    ):
        message = await bot.send_animation(
            chat_id,
            animation_file,
            duration=duration,
            width=self.width,
            height=self.height,
            caption=self.caption,
            parse_mode="Markdown",
            disable_notification=False,
            protect_content=True,
            thumbnail=thumb_file,
            has_spoiler=True,
            show_caption_above_media=True,
        )

        assert isinstance(message.animation, Animation)
        assert isinstance(message.animation.file_id, str)
        assert isinstance(message.animation.file_unique_id, str)
        assert message.animation.file_id
        assert message.animation.file_unique_id
        assert message.animation.file_name == animation.file_name
        assert message.animation.mime_type == animation.mime_type
        # TGs reported file size is not reliable
        assert isinstance(message.animation.file_size, int)
        assert message.animation.thumbnail.width == self.width
        assert message.animation.thumbnail.height == self.height
        assert message.has_protected_content
        assert message.show_caption_above_media
        try:
            assert message.has_media_spoiler
        except AssertionError:
            pytest.xfail("This is a bug on Telegram's end")

    async def test_get_and_download(self, bot, animation, tmp_file):
        new_file = await bot.get_file(animation.file_id)

        assert new_file.file_path.startswith("https://")

        new_filepath = await new_file.download_to_drive(tmp_file)
        assert new_filepath.is_file()

    async def test_send_animation_url_file(self, bot, chat_id, animation):
        message = await bot.send_animation(
            chat_id=chat_id, animation=self.animation_file_url, caption=self.caption
        )

        assert message.caption == self.caption

        assert isinstance(message.animation, Animation)
        assert isinstance(message.animation.file_id, str)
        assert isinstance(message.animation.file_unique_id, str)
        assert message.animation.file_id
        assert message.animation.file_unique_id

        assert message.animation.duration == animation.duration
        assert message.animation.file_name.startswith(
            "game.gif"
        ) == animation.file_name.startswith("game.gif")
        assert message.animation.mime_type == animation.mime_type

    async def test_send_animation_caption_entities(self, bot, chat_id, animation):
        test_string = "Italic Bold Code"
        entities = [
            MessageEntity(MessageEntity.ITALIC, 0, 6),
            MessageEntity(MessageEntity.ITALIC, 7, 4),
            MessageEntity(MessageEntity.ITALIC, 12, 4),
        ]
        message = await bot.send_animation(
            chat_id, animation, caption=test_string, caption_entities=entities
        )

        assert message.caption == test_string
        assert message.caption_entities == tuple(entities)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_animation_default_parse_mode_1(self, default_bot, chat_id, animation_file):
        test_string = "Italic Bold Code"
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_animation(
            chat_id, animation_file, caption=test_markdown_string
        )
        assert message.caption_markdown == test_markdown_string
        assert message.caption == test_string

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_animation_default_parse_mode_2(self, default_bot, chat_id, animation_file):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_animation(
            chat_id, animation_file, caption=test_markdown_string, parse_mode=None
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_animation_default_parse_mode_3(self, default_bot, chat_id, animation_file):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_animation(
            chat_id, animation_file, caption=test_markdown_string, parse_mode="HTML"
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"allow_sending_without_reply": True}, None),
            ({"allow_sending_without_reply": False}, None),
            ({"allow_sending_without_reply": False}, True),
        ],
        indirect=["default_bot"],
    )
    async def test_send_animation_default_allow_sending_without_reply(
        self, default_bot, chat_id, animation, custom
    ):
        reply_to_message = await default_bot.send_message(chat_id, "test")
        await reply_to_message.delete()
        if custom is not None:
            message = await default_bot.send_animation(
                chat_id,
                animation,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        elif default_bot.defaults.allow_sending_without_reply:
            message = await default_bot.send_animation(
                chat_id, animation, reply_to_message_id=reply_to_message.message_id
            )
            assert message.reply_to_message is None
        else:
            with pytest.raises(BadRequest, match="Message to be replied not found"):
                await default_bot.send_animation(
                    chat_id, animation, reply_to_message_id=reply_to_message.message_id
                )

    @pytest.mark.parametrize("default_bot", [{"protect_content": True}], indirect=True)
    async def test_send_animation_default_protect_content(self, default_bot, chat_id, animation):
        tasks = asyncio.gather(
            default_bot.send_animation(chat_id, animation),
            default_bot.send_animation(chat_id, animation, protect_content=False),
        )
        anim_protected, anim_unprotected = await tasks
        assert anim_protected.has_protected_content
        assert not anim_unprotected.has_protected_content

    async def test_resend(self, bot, chat_id, animation):
        message = await bot.send_animation(chat_id, animation.file_id)
        assert message.animation == animation

    async def test_error_send_empty_file(self, bot, chat_id):
        with Path(os.devnull).open("rb") as animation_file, pytest.raises(TelegramError):
            await bot.send_animation(chat_id=chat_id, animation=animation_file)

    async def test_error_send_empty_file_id(self, bot, chat_id):
        with pytest.raises(TelegramError):
            await bot.send_animation(chat_id=chat_id, animation="")

    async def test_error_send_without_required_args(self, bot, chat_id):
        with pytest.raises(TypeError):
            await bot.send_animation(chat_id=chat_id)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import LabeledPrice, Location
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def labeled_price():
    return LabeledPrice(LabeledPriceTestBase.label, LabeledPriceTestBase.amount)


class LabeledPriceTestBase:
    label = "label"
    amount = 100


class TestLabeledPriceWithoutRequest(LabeledPriceTestBase):
    def test_slot_behaviour(self, labeled_price):
        inst = labeled_price
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, labeled_price):
        assert labeled_price.label == self.label
        assert labeled_price.amount == self.amount

    def test_to_dict(self, labeled_price):
        labeled_price_dict = labeled_price.to_dict()

        assert isinstance(labeled_price_dict, dict)
        assert labeled_price_dict["label"] == labeled_price.label
        assert labeled_price_dict["amount"] == labeled_price.amount

    def test_equality(self):
        a = LabeledPrice("label", 100)
        b = LabeledPrice("label", 100)
        c = LabeledPrice("Label", 101)
        d = Location(123, 456)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Common base class for media objects with thumbnails"""

from typing import TYPE_CHECKING, TypeVar

from telegram._files._basemedium import _BaseMedium
from telegram._files.photosize import PhotoSize
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot

# pylint: disable=invalid-name
ThumbedMT_co = TypeVar("ThumbedMT_co", bound="_BaseThumbedMedium", covariant=True)


class _BaseThumbedMedium(_BaseMedium):
    """
    Base class for objects representing the various media file types that may include a thumbnail.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`file_unique_id` is equal.

    Args:
        file_id (:obj:`str`): Identifier for this file, which can be used to download
            or reuse the file.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`, optional): File size.
        thumbnail (:class:`telegram.PhotoSize`, optional): Thumbnail as defined by the sender.

            .. versionadded:: 20.2

    Attributes:
        file_id (:obj:`str`): File identifier.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`): Optional. File size.
        thumbnail (:class:`telegram.PhotoSize`): Optional. Thumbnail as defined by the sender.

            .. versionadded:: 20.2

    """

    __slots__ = ("thumbnail",)

    def __init__(
        self,
        file_id: str,
        file_unique_id: str,
        file_size: int | None = None,
        thumbnail: PhotoSize | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(
            file_id=file_id,
            file_unique_id=file_unique_id,
            file_size=file_size,
            api_kwargs=api_kwargs,
        )

        self.thumbnail: PhotoSize | None = thumbnail

    @classmethod
    def de_json(cls: type[ThumbedMT_co], data: JSONDict, bot: "Bot | None" = None) -> ThumbedMT_co:
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # In case this wasn't already done by the subclass
        if not isinstance(data.get("thumbnail"), PhotoSize):
            data["thumbnail"] = de_json_optional(data.get("thumbnail"), PhotoSize, bot)

        api_kwargs = {}
        # This is a deprecated field that TG still returns for backwards compatibility
        # Let's filter it out to speed up the de-json process
        if data.get("thumb") is not None:
            api_kwargs["thumb"] = data.pop("thumb")

        return super()._de_json(data=data, bot=bot, api_kwargs=api_kwargs)



#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

from telegram import MessageAutoDeleteTimerChanged, VideoChatEnded
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.slots import mro_slots


class TestMessageAutoDeleteTimerChangedWithoutRequest:
    message_auto_delete_time = dtm.timedelta(seconds=100)

    def test_slot_behaviour(self):
        action = MessageAutoDeleteTimerChanged(self.message_auto_delete_time)
        for attr in action.__slots__:
            assert getattr(action, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(action)) == len(set(mro_slots(action))), "duplicate slot"

    def test_de_json(self):
        json_dict = {
            "message_auto_delete_time": int(self.message_auto_delete_time.total_seconds())
        }
        madtc = MessageAutoDeleteTimerChanged.de_json(json_dict, None)
        assert madtc.api_kwargs == {}

        assert madtc._message_auto_delete_time == self.message_auto_delete_time

    def test_to_dict(self):
        madtc = MessageAutoDeleteTimerChanged(self.message_auto_delete_time)
        madtc_dict = madtc.to_dict()

        assert isinstance(madtc_dict, dict)
        assert madtc_dict["message_auto_delete_time"] == int(
            self.message_auto_delete_time.total_seconds()
        )
        assert isinstance(madtc_dict["message_auto_delete_time"], int)

    def test_time_period_properties(self, PTB_TIMEDELTA):
        message_auto_delete_time = MessageAutoDeleteTimerChanged(
            self.message_auto_delete_time
        ).message_auto_delete_time

        if PTB_TIMEDELTA:
            assert message_auto_delete_time == self.message_auto_delete_time
            assert isinstance(message_auto_delete_time, dtm.timedelta)
        else:
            assert message_auto_delete_time == int(self.message_auto_delete_time.total_seconds())
            assert isinstance(message_auto_delete_time, int)

    def test_time_period_int_deprecated(self, recwarn, PTB_TIMEDELTA):
        MessageAutoDeleteTimerChanged(self.message_auto_delete_time).message_auto_delete_time

        if PTB_TIMEDELTA:
            assert len(recwarn) == 0
        else:
            assert len(recwarn) == 1
            assert "`message_auto_delete_time` will be of type `datetime.timedelta`" in str(
                recwarn[0].message
            )
            assert recwarn[0].category is PTBDeprecationWarning

    def test_equality(self):
        a = MessageAutoDeleteTimerChanged(100)
        b = MessageAutoDeleteTimerChanged(100)
        c = MessageAutoDeleteTimerChanged(50)
        d = VideoChatEnded(25)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
"""Simple Bot to reply to Telegram messages.

This is built on the API wrapper, see echobot.py to see the same example built
on the telegram.ext bot framework.
This program is dedicated to the public domain under the CC0 license.
"""

import asyncio
import contextlib
import datetime as dtm
import logging
from typing import NoReturn

from telegram import Bot, Update
from telegram.error import Forbidden, NetworkError

logging.basicConfig(
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
)
# set higher logging level for httpx to avoid all GET and POST requests being logged
logging.getLogger("httpx").setLevel(logging.WARNING)

logger = logging.getLogger(__name__)


async def main() -> NoReturn:
    """Run the bot."""
    # Here we use the `async with` syntax to properly initialize and shutdown resources.
    async with Bot("TOKEN") as bot:
        # get the first pending update_id, this is so we can skip over it in case
        # we get a "Forbidden" exception.
        try:
            update_id = (await bot.get_updates())[0].update_id
        except IndexError:
            update_id = None

        logger.info("listening for new messages...")
        while True:
            try:
                update_id = await echo(bot, update_id)
            except NetworkError:
                await asyncio.sleep(1)
            except Forbidden:
                # The user has removed or blocked the bot.
                update_id += 1


async def echo(bot: Bot, update_id: int) -> int:
    """Echo the message the user sent."""
    # Request updates after the last update_id
    updates = await bot.get_updates(
        offset=update_id, timeout=dtm.timedelta(seconds=10), allowed_updates=Update.ALL_TYPES
    )
    for update in updates:
        next_update_id = update.update_id + 1

        # your bot can receive updates without messages
        # and not all messages contain text
        if update.message and update.message.text:
            # Reply to the message
            logger.info("Found message %s!", update.message.text)
            await update.message.reply_text(update.message.text)
        return next_update_id
    return update_id


if __name__ == "__main__":
    with contextlib.suppress(KeyboardInterrupt):  # Ignore exception when Ctrl-C is pressed
        asyncio.run(main())



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains helper functions related to parsing arguments for classes and methods.

Warning:
    Contents of this module are intended to be used internally by the library and *not* by the
    user. Changes to this module are not considered breaking changes and may not be documented in
    the changelog.
"""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING, Protocol, TypeVar, overload

from telegram._linkpreviewoptions import LinkPreviewOptions
from telegram._telegramobject import TelegramObject
from telegram._utils.types import JSONDict, ODVInput

if TYPE_CHECKING:
    from typing import type_check_only

    from telegram import Bot, FileCredentials

T = TypeVar("T")


def parse_sequence_arg(arg: Sequence[T] | None) -> tuple[T, ...]:
    """Parses an optional sequence into a tuple

    Args:
        arg (:obj:`Sequence`): The sequence to parse.

    Returns:
        :obj:`Tuple`: The sequence converted to a tuple or an empty tuple.
    """
    return tuple(arg) if arg else ()


@overload
def to_timedelta(arg: None) -> None: ...


@overload
def to_timedelta(
    arg: (
        int | float | dtm.timedelta  # noqa: PYI041 (be more explicit about `int` and `float` args)
    ),
) -> dtm.timedelta: ...


def to_timedelta(arg: int | float | dtm.timedelta | None) -> dtm.timedelta | None:  # noqa: PYI041
    """Parses an optional time period in seconds into a timedelta

    Args:
        arg (:obj:`int` | :class:`datetime.timedelta`, optional): The time period to parse.

    Returns:
        :obj:`timedelta`: The time period converted to a timedelta object or :obj:`None`.
    """
    if arg is None:
        return None
    if isinstance(arg, int | float):
        return dtm.timedelta(seconds=arg)
    return arg


def parse_lpo_and_dwpp(
    disable_web_page_preview: bool | None, link_preview_options: ODVInput[LinkPreviewOptions]
) -> ODVInput[LinkPreviewOptions]:
    """Wrapper around warn_about_deprecated_arg_return_new_arg. Takes care of converting
    disable_web_page_preview to LinkPreviewOptions.
    """
    if disable_web_page_preview and link_preview_options:
        raise ValueError(
            "Parameters `disable_web_page_preview` and `link_preview_options` are mutually "
            "exclusive."
        )

    if disable_web_page_preview is not None:
        link_preview_options = LinkPreviewOptions(is_disabled=disable_web_page_preview)

    return link_preview_options


Tele_co = TypeVar("Tele_co", bound=TelegramObject, covariant=True)
TeleCrypto_co = TypeVar("TeleCrypto_co", bound="HasDecryptMethod", covariant=True)

if TYPE_CHECKING:

    @type_check_only
    class HasDecryptMethod(Protocol):
        __slots__ = ()

        @classmethod
        def de_json_decrypted(
            cls: type[TeleCrypto_co],
            data: JSONDict,
            bot: "Bot | None",
            credentials: list["FileCredentials"],
        ) -> TeleCrypto_co: ...

        @classmethod
        def de_list_decrypted(
            cls: type[TeleCrypto_co],
            data: list[JSONDict],
            bot: "Bot | None",
            credentials: list["FileCredentials"],
        ) -> tuple[TeleCrypto_co, ...]: ...


def de_json_optional(
    data: JSONDict | None, cls: type[Tele_co], bot: "Bot | None"
) -> Tele_co | None:
    """Wrapper around TO.de_json that returns None if data is None."""
    if data is None:
        return None

    return cls.de_json(data, bot)


def de_json_decrypted_optional(
    data: JSONDict | None,
    cls: type[TeleCrypto_co],
    bot: "Bot | None",
    credentials: list["FileCredentials"],
) -> TeleCrypto_co | None:
    """Wrapper around TO.de_json_decrypted that returns None if data is None."""
    if data is None:
        return None

    return cls.de_json_decrypted(data, bot, credentials)


def de_list_optional(
    data: list[JSONDict] | None, cls: type[Tele_co], bot: "Bot | None"
) -> tuple[Tele_co, ...]:
    """Wrapper around TO.de_list that returns an empty list if data is None."""
    if data is None:
        return ()

    return cls.de_list(data, bot)


def de_list_decrypted_optional(
    data: list[JSONDict] | None,
    cls: type[TeleCrypto_co],
    bot: "Bot | None",
    credentials: list["FileCredentials"],
) -> tuple[TeleCrypto_co, ...]:
    """Wrapper around TO.de_list_decrypted that returns an empty list if data is None."""
    if data is None:
        return ()

    return cls.de_list_decrypted(data, bot, credentials)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import asyncio

import pytest

from telegram import (
    Bot,
    CallbackQuery,
    Chat,
    ChosenInlineResult,
    InlineQuery,
    Location,
    Message,
    PreCheckoutQuery,
    ShippingQuery,
    Update,
    User,
)
from telegram.ext import CallbackContext, InlineQueryHandler, JobQueue
from tests.auxil.slots import mro_slots

message = Message(1, None, Chat(1, ""), from_user=User(1, "", False), text="Text")

params = [
    {"message": message},
    {"edited_message": message},
    {"callback_query": CallbackQuery(1, User(1, "", False), "chat", message=message)},
    {"channel_post": message},
    {"edited_channel_post": message},
    {"chosen_inline_result": ChosenInlineResult("id", User(1, "", False), "")},
    {"shipping_query": ShippingQuery("id", User(1, "", False), "", None)},
    {"pre_checkout_query": PreCheckoutQuery("id", User(1, "", False), "", 0, "")},
    {"callback_query": CallbackQuery(1, User(1, "", False), "chat")},
]

ids = (
    "message",
    "edited_message",
    "callback_query",
    "channel_post",
    "edited_channel_post",
    "chosen_inline_result",
    "shipping_query",
    "pre_checkout_query",
    "callback_query_without_message",
)


@pytest.fixture(scope="class", params=params, ids=ids)
def false_update(request):
    return Update(update_id=2, **request.param)


@pytest.fixture
def inline_query(bot):
    update = Update(
        0,
        inline_query=InlineQuery(
            "id",
            User(2, "test user", False),
            "test query",
            offset="22",
            location=Location(latitude=-23.691288, longitude=-46.788279),
        ),
    )
    update._unfreeze()
    update.inline_query._unfreeze()
    return update


class TestInlineQueryHandler:
    test_flag = False

    def test_slot_behaviour(self):
        handler = InlineQueryHandler(self.callback)
        for attr in handler.__slots__:
            assert getattr(handler, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(handler)) == len(set(mro_slots(handler))), "duplicate slot"

    @pytest.fixture(autouse=True)
    def _reset(self):
        self.test_flag = False

    async def callback(self, update, context):
        self.test_flag = (
            isinstance(context, CallbackContext)
            and isinstance(context.bot, Bot)
            and isinstance(update, Update)
            and isinstance(context.update_queue, asyncio.Queue)
            and isinstance(context.job_queue, JobQueue)
            and isinstance(context.user_data, dict)
            and context.chat_data is None
            and isinstance(context.bot_data, dict)
            and isinstance(update.inline_query, InlineQuery)
        )

    def callback_pattern(self, update, context):
        if context.matches[0].groups():
            self.test_flag = context.matches[0].groups() == ("t", " query")
        if context.matches[0].groupdict():
            self.test_flag = context.matches[0].groupdict() == {"begin": "t", "end": " query"}

    def test_other_update_types(self, false_update):
        handler = InlineQueryHandler(self.callback)
        assert not handler.check_update(false_update)

    async def test_context(self, app, inline_query):
        handler = InlineQueryHandler(self.callback)
        app.add_handler(handler)

        async with app:
            await app.process_update(inline_query)
        assert self.test_flag

    async def test_context_pattern(self, app, inline_query):
        handler = InlineQueryHandler(self.callback_pattern, pattern=r"(?P<begin>.*)est(?P<end>.*)")
        app.add_handler(handler)

        async with app:
            await app.process_update(inline_query)
            assert self.test_flag

            app.remove_handler(handler)
            handler = InlineQueryHandler(self.callback_pattern, pattern=r"(t)est(.*)")
            app.add_handler(handler)

            await app.process_update(inline_query)
            assert self.test_flag

            update = Update(
                update_id=0, inline_query=InlineQuery(id="id", from_user=None, query="", offset="")
            )
            update.inline_query._unfreeze()
            assert not handler.check_update(update)
            update.inline_query.query = "not_a_match"
            assert not handler.check_update(update)

    @pytest.mark.parametrize(
        ("query", "expected_result"),
        [
            pytest.param("", True, id="empty string"),
            pytest.param("not empty", False, id="non_empty_string"),
        ],
    )
    async def test_empty_inline_query_pattern(self, app, query, expected_result):
        handler = InlineQueryHandler(self.callback, pattern=r"^$")
        app.add_handler(handler)

        update = Update(
            update_id=0,
            inline_query=InlineQuery(
                id="id", from_user=User(1, "test", False), query=query, offset=""
            ),
        )

        async with app:
            await app.process_update(update)
        assert self.test_flag == expected_result

    @pytest.mark.parametrize("chat_types", [[Chat.SENDER], [Chat.SENDER, Chat.SUPERGROUP], []])
    @pytest.mark.parametrize(
        ("chat_type", "result"), [(Chat.SENDER, True), (Chat.CHANNEL, False), (None, False)]
    )
    async def test_chat_types(self, app, inline_query, chat_types, chat_type, result):
        try:
            inline_query.inline_query.chat_type = chat_type

            handler = InlineQueryHandler(self.callback, chat_types=chat_types)
            app.add_handler(handler)
            async with app:
                await app.process_update(inline_query)

            if not chat_types:
                assert self.test_flag is False
            else:
                assert self.test_flag == result

        finally:
            inline_query.inline_query.chat_type = None



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an objects that are related to Telegram giveaways."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.datetime import extract_tzinfo_from_defaults, from_timestamp
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot, Message


class Giveaway(TelegramObject):
    """This object represents a message about a scheduled giveaway.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`chats`, :attr:`winners_selection_date` and
    :attr:`winner_count` are equal.

    .. versionadded:: 20.8

    Args:
        chats (tuple[:class:`telegram.Chat`]): The list of chats which the user must join to
            participate in the giveaway.
        winners_selection_date (:class:`datetime.datetime`): The date when the giveaway winner will
            be selected. |datetime_localization|
        winner_count (:obj:`int`): The number of users which are supposed to be selected as winners
            of the giveaway.
        only_new_members (:obj:`True`, optional): If :obj:`True`, only users who join the chats
            after the giveaway started should be eligible to win.
        has_public_winners (:obj:`True`, optional): :obj:`True`, if the list of giveaway winners
            will be visible to everyone
        prize_description (:obj:`str`, optional): Description of additional giveaway prize
        country_codes (Sequence[:obj:`str`]): A list of two-letter ISO 3166-1 alpha-2
            country codes indicating the countries from which eligible users for the giveaway must
            come. If empty, then all users can participate in the giveaway. Users with a phone
            number that was bought on Fragment can always participate in giveaways.
        prize_star_count (:obj:`int`, optional): The number of Telegram Stars to be split between
            giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6
        premium_subscription_month_count (:obj:`int`, optional): The number of months the Telegram
            Premium subscription won from the giveaway will be active for; for Telegram Premium
            giveaways only.

    Attributes:
        chats (Sequence[:class:`telegram.Chat`]): The list of chats which the user must join to
            participate in the giveaway.
        winners_selection_date (:class:`datetime.datetime`): The date when the giveaway winner will
            be selected. |datetime_localization|
        winner_count (:obj:`int`): The number of users which are supposed to be selected as winners
            of the giveaway.
        only_new_members (:obj:`True`): Optional. If :obj:`True`, only users who join the chats
            after the giveaway started should be eligible to win.
        has_public_winners (:obj:`True`): Optional. :obj:`True`, if the list of giveaway winners
            will be visible to everyone
        prize_description (:obj:`str`): Optional. Description of additional giveaway prize
        country_codes (tuple[:obj:`str`]): Optional. A tuple of two-letter ISO 3166-1 alpha-2
            country codes indicating the countries from which eligible users for the giveaway must
            come. If empty, then all users can participate in the giveaway. Users with a phone
            number that was bought on Fragment can always participate in giveaways.
        prize_star_count (:obj:`int`): Optional. The number of Telegram Stars to be split between
            giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6
        premium_subscription_month_count (:obj:`int`): Optional. The number of months the Telegram
            Premium subscription won from the giveaway will be active for; for Telegram Premium
            giveaways only.
    """

    __slots__ = (
        "chats",
        "country_codes",
        "has_public_winners",
        "only_new_members",
        "premium_subscription_month_count",
        "prize_description",
        "prize_star_count",
        "winner_count",
        "winners_selection_date",
    )

    def __init__(
        self,
        chats: Sequence[Chat],
        winners_selection_date: dtm.datetime,
        winner_count: int,
        only_new_members: bool | None = None,
        has_public_winners: bool | None = None,
        prize_description: str | None = None,
        country_codes: Sequence[str] | None = None,
        premium_subscription_month_count: int | None = None,
        prize_star_count: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.chats: tuple[Chat, ...] = tuple(chats)
        self.winners_selection_date: dtm.datetime = winners_selection_date
        self.winner_count: int = winner_count
        self.only_new_members: bool | None = only_new_members
        self.has_public_winners: bool | None = has_public_winners
        self.prize_description: str | None = prize_description
        self.country_codes: tuple[str, ...] = parse_sequence_arg(country_codes)
        self.premium_subscription_month_count: int | None = premium_subscription_month_count
        self.prize_star_count: int | None = prize_star_count

        self._id_attrs = (
            self.chats,
            self.winners_selection_date,
            self.winner_count,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "Giveaway":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["chats"] = de_list_optional(data.get("chats"), Chat, bot)
        data["winners_selection_date"] = from_timestamp(
            data.get("winners_selection_date"), tzinfo=loc_tzinfo
        )

        return super().de_json(data=data, bot=bot)


class GiveawayCreated(TelegramObject):
    """This object represents a service message about the creation of a scheduled giveaway.

    Args:
        prize_star_count (:obj:`int`, optional): The number of Telegram Stars to be
            split between giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6

    Attributes:
        prize_star_count (:obj:`int`): Optional. The number of Telegram Stars to be
            split between giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6

    """

    __slots__ = ("prize_star_count",)

    def __init__(self, prize_star_count: int | None = None, *, api_kwargs: JSONDict | None = None):
        super().__init__(api_kwargs=api_kwargs)
        self.prize_star_count: int | None = prize_star_count

        self._freeze()


class GiveawayWinners(TelegramObject):
    """This object represents a message about the completion of a giveaway with public winners.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`chat`, :attr:`giveaway_message_id`,
    :attr:`winners_selection_date`, :attr:`winner_count` and :attr:`winners` are equal.

    .. versionadded:: 20.8

    Args:
        chat (:class:`telegram.Chat`): The chat that created the giveaway
        giveaway_message_id (:obj:`int`): Identifier of the message with the giveaway in the chat
        winners_selection_date (:class:`datetime.datetime`): Point in time when winners of the
            giveaway were selected. |datetime_localization|
        winner_count (:obj:`int`): Total number of winners in the giveaway
        winners	(Sequence[:class:`telegram.User`]): List of up to
            :tg-const:`telegram.constants.GiveawayLimit.MAX_WINNERS` winners of the giveaway
        prize_star_count (:obj:`int`, optional): The number of Telegram Stars to be split between
            giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6
        additional_chat_count (:obj:`int`, optional): The number of other chats the user had to
            join in order to be eligible for the giveaway
        premium_subscription_month_count (:obj:`int`, optional): The number of months the Telegram
            Premium subscription won from the giveaway will be active for
        unclaimed_prize_count (:obj:`int`, optional): Number of undistributed prizes
        only_new_members (:obj:`True`, optional): :obj:`True`, if only users who had joined the
            chats after the giveaway started were eligible to win
        was_refunded (:obj:`True`, optional): :obj:`True`, if the giveaway was canceled because the
            payment for it was refunded
        prize_description (:obj:`str`, optional): Description of additional giveaway prize

    Attributes:
        chat (:class:`telegram.Chat`): The chat that created the giveaway
        giveaway_message_id (:obj:`int`): Identifier of the message with the giveaway in the chat
        winners_selection_date (:class:`datetime.datetime`): Point in time when winners of the
            giveaway were selected. |datetime_localization|
        winner_count (:obj:`int`): Total number of winners in the giveaway
        winners	(tuple[:class:`telegram.User`]): tuple of up to
            :tg-const:`telegram.constants.GiveawayLimit.MAX_WINNERS` winners of the giveaway
        additional_chat_count (:obj:`int`): Optional. The number of other chats the user had to
            join in order to be eligible for the giveaway
        prize_star_count (:obj:`int`): Optional. The number of Telegram Stars to be split between
            giveaway winners; for Telegram Star giveaways only.

            .. versionadded:: 21.6
        premium_subscription_month_count (:obj:`int`): Optional. The number of months the Telegram
            Premium subscription won from the giveaway will be active for
        unclaimed_prize_count (:obj:`int`): Optional. Number of undistributed prizes
        only_new_members (:obj:`True`): Optional. :obj:`True`, if only users who had joined the
            chats after the giveaway started were eligible to win
        was_refunded (:obj:`True`): Optional. :obj:`True`, if the giveaway was canceled because the
            payment for it was refunded
        prize_description (:obj:`str`): Optional. Description of additional giveaway prize
    """

    __slots__ = (
        "additional_chat_count",
        "chat",
        "giveaway_message_id",
        "only_new_members",
        "premium_subscription_month_count",
        "prize_description",
        "prize_star_count",
        "unclaimed_prize_count",
        "was_refunded",
        "winner_count",
        "winners",
        "winners_selection_date",
    )

    def __init__(
        self,
        chat: Chat,
        giveaway_message_id: int,
        winners_selection_date: dtm.datetime,
        winner_count: int,
        winners: Sequence[User],
        additional_chat_count: int | None = None,
        premium_subscription_month_count: int | None = None,
        unclaimed_prize_count: int | None = None,
        only_new_members: bool | None = None,
        was_refunded: bool | None = None,
        prize_description: str | None = None,
        prize_star_count: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.chat: Chat = chat
        self.giveaway_message_id: int = giveaway_message_id
        self.winners_selection_date: dtm.datetime = winners_selection_date
        self.winner_count: int = winner_count
        self.winners: tuple[User, ...] = tuple(winners)
        self.additional_chat_count: int | None = additional_chat_count
        self.premium_subscription_month_count: int | None = premium_subscription_month_count
        self.unclaimed_prize_count: int | None = unclaimed_prize_count
        self.only_new_members: bool | None = only_new_members
        self.was_refunded: bool | None = was_refunded
        self.prize_description: str | None = prize_description
        self.prize_star_count: int | None = prize_star_count

        self._id_attrs = (
            self.chat,
            self.giveaway_message_id,
            self.winners_selection_date,
            self.winner_count,
            self.winners,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "GiveawayWinners":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["chat"] = de_json_optional(data.get("chat"), Chat, bot)
        data["winners"] = de_list_optional(data.get("winners"), User, bot)
        data["winners_selection_date"] = from_timestamp(
            data.get("winners_selection_date"), tzinfo=loc_tzinfo
        )

        return super().de_json(data=data, bot=bot)


class GiveawayCompleted(TelegramObject):
    """This object represents a service message about the completion of a giveaway without public
    winners.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`winner_count` and :attr:`unclaimed_prize_count` are equal.

    .. versionadded:: 20.8


    Args:
        winner_count (:obj:`int`): Number of winners in the giveaway
        unclaimed_prize_count (:obj:`int`, optional): Number of undistributed prizes
        giveaway_message (:class:`telegram.Message`, optional): Message with the giveaway that was
            completed, if it wasn't deleted
        is_star_giveaway (:obj:`bool`, optional): :obj:`True`, if the giveaway is a Telegram Star
            giveaway. Otherwise, currently, the giveaway is a Telegram Premium giveaway.

            .. versionadded:: 21.6
    Attributes:
        winner_count (:obj:`int`): Number of winners in the giveaway
        unclaimed_prize_count (:obj:`int`): Optional. Number of undistributed prizes
        giveaway_message (:class:`telegram.Message`): Optional. Message with the giveaway that was
            completed, if it wasn't deleted
        is_star_giveaway (:obj:`bool`): Optional. :obj:`True`, if the giveaway is a Telegram Star
            giveaway. Otherwise, currently, the giveaway is a Telegram Premium giveaway.

            .. versionadded:: 21.6
    """

    __slots__ = ("giveaway_message", "is_star_giveaway", "unclaimed_prize_count", "winner_count")

    def __init__(
        self,
        winner_count: int,
        unclaimed_prize_count: int | None = None,
        giveaway_message: "Message | None" = None,
        is_star_giveaway: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        self.winner_count: int = winner_count
        self.unclaimed_prize_count: int | None = unclaimed_prize_count
        self.giveaway_message: Message | None = giveaway_message
        self.is_star_giveaway: bool | None = is_star_giveaway

        self._id_attrs = (
            self.winner_count,
            self.unclaimed_prize_count,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "GiveawayCompleted":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Unfortunately, this needs to be here due to cyclic imports
        from telegram._message import (  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
            Message,
        )

        data["giveaway_message"] = de_json_optional(data.get("giveaway_message"), Message, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram UserProfileAudios."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._files.audio import Audio
from telegram._telegramobject import TelegramObject
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class UserProfileAudios(TelegramObject):
    """
    This object represents the audios displayed on a user's profile.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`total_count` and :attr:`audios` are equal.

    .. versionadded:: 22.7

    Args:
        total_count (:obj:`int`): Total number of profile audios for the target user.
        audios (Sequence[:class:`telegram.Audio`]): Requested profile audios.

    Attributes:
        total_count (:obj:`int`): Total number of profile audios for the target user.
        audios (tuple[:class:`telegram.Audio`]): Requested profile audios.

    """

    __slots__ = ("audios", "total_count")

    def __init__(
        self,
        total_count: int,
        audios: Sequence[Audio],
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.total_count: int = total_count
        self.audios: tuple[Audio, ...] = tuple(audios)

        self._id_attrs = (self.total_count, self.audios)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "UserProfileAudios":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["audios"] = Audio.de_list(data.get("audios", []), bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import BotCommand, ProximityAlertTriggered, User
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def proximity_alert_triggered():
    return ProximityAlertTriggered(
        ProximityAlertTriggeredTestBase.traveler,
        ProximityAlertTriggeredTestBase.watcher,
        ProximityAlertTriggeredTestBase.distance,
    )


class ProximityAlertTriggeredTestBase:
    traveler = User(1, "foo", False)
    watcher = User(2, "bar", False)
    distance = 42


class TestProximityAlertTriggeredWithoutRequest(ProximityAlertTriggeredTestBase):
    def test_slot_behaviour(self, proximity_alert_triggered):
        inst = proximity_alert_triggered
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "traveler": self.traveler.to_dict(),
            "watcher": self.watcher.to_dict(),
            "distance": self.distance,
        }
        proximity_alert_triggered = ProximityAlertTriggered.de_json(json_dict, offline_bot)
        assert proximity_alert_triggered.api_kwargs == {}

        assert proximity_alert_triggered.traveler == self.traveler
        assert proximity_alert_triggered.traveler.first_name == self.traveler.first_name
        assert proximity_alert_triggered.watcher == self.watcher
        assert proximity_alert_triggered.watcher.first_name == self.watcher.first_name
        assert proximity_alert_triggered.distance == self.distance

    def test_to_dict(self, proximity_alert_triggered):
        proximity_alert_triggered_dict = proximity_alert_triggered.to_dict()

        assert isinstance(proximity_alert_triggered_dict, dict)
        assert (
            proximity_alert_triggered_dict["traveler"]
            == proximity_alert_triggered.traveler.to_dict()
        )
        assert (
            proximity_alert_triggered_dict["watcher"]
            == proximity_alert_triggered.watcher.to_dict()
        )
        assert proximity_alert_triggered_dict["distance"] == proximity_alert_triggered.distance

    def test_equality(self, proximity_alert_triggered):
        a = proximity_alert_triggered
        b = ProximityAlertTriggered(User(1, "John", False), User(2, "Doe", False), 42)
        c = ProximityAlertTriggered(User(3, "John", False), User(2, "Doe", False), 42)
        d = ProximityAlertTriggered(User(1, "John", False), User(3, "Doe", False), 42)
        e = ProximityAlertTriggered(User(1, "John", False), User(2, "Doe", False), 43)
        f = BotCommand("start", "description")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from collections import defaultdict
from pathlib import Path

import pytest

from telegram._utils.warnings import warn
from telegram.warnings import PTBDeprecationWarning, PTBRuntimeWarning, PTBUserWarning
from tests.auxil.files import SOURCE_ROOT_PATH
from tests.auxil.slots import mro_slots


class TestWarnings:
    @pytest.mark.parametrize(
        "inst",
        [
            (PTBUserWarning("test message")),
            (PTBRuntimeWarning("test message")),
            (PTBDeprecationWarning("20.6", "test message")),
        ],
    )
    def test_slots_behavior(self, inst):
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_test_coverage(self):
        """This test is only here to make sure that new warning classes will set __slots__
        properly.
        Add the new warning class to the below covered_subclasses dict, if it's covered in the
        above test_slots_behavior tests.
        """

        def make_assertion(cls):
            assert set(cls.__subclasses__()) == covered_subclasses[cls]
            for subcls in cls.__subclasses__():
                make_assertion(subcls)

        covered_subclasses = defaultdict(set)
        covered_subclasses.update(
            {
                PTBUserWarning: {
                    PTBRuntimeWarning,
                    PTBDeprecationWarning,
                },
            }
        )

        make_assertion(PTBUserWarning)

    def test_warn(self, recwarn):
        expected_file = SOURCE_ROOT_PATH / "_utils" / "warnings.py"

        warn("test message")
        assert len(recwarn) == 1
        assert recwarn[0].category is PTBUserWarning
        assert str(recwarn[0].message) == "test message"
        assert Path(recwarn[0].filename) == expected_file, "incorrect stacklevel!"

        warn("test message 2", category=PTBRuntimeWarning)
        assert len(recwarn) == 2
        assert recwarn[1].category is PTBRuntimeWarning
        assert str(recwarn[1].message) == "test message 2"
        assert Path(recwarn[1].filename) == expected_file, "incorrect stacklevel!"

        warn(PTBDeprecationWarning("20.6", "test message 3"), stacklevel=1)
        assert len(recwarn) == 3
        assert recwarn[2].category is PTBDeprecationWarning
        assert str(recwarn[2].message) == "Deprecated since version 20.6: test message 3"
        assert Path(recwarn[2].filename) == Path(__file__), "incorrect stacklevel!"



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
# pylint: disable=missing-module-docstring
import asyncio
import json
from http import HTTPStatus
from pathlib import Path
from socket import socket
from ssl import SSLContext
from types import TracebackType
from typing import TYPE_CHECKING

# Instead of checking for ImportError here, we do that in `updater.py`, where we import from
# this module. Doing it here would be tricky, as the classes below subclass tornado classes
import tornado.web
from tornado.httpserver import HTTPServer

try:
    from tornado.netutil import bind_unix_socket

    UNIX_AVAILABLE = True
except ImportError:
    UNIX_AVAILABLE = False

from telegram import Update
from telegram._utils.logging import get_logger
from telegram.ext._extbot import ExtBot

if TYPE_CHECKING:
    from telegram import Bot

# This module is not visible to users, so we log as Updater
_LOGGER = get_logger(__name__, class_name="Updater")


class WebhookServer:
    """Thin wrapper around ``tornado.httpserver.HTTPServer``."""

    __slots__ = (
        "_http_server",
        "_server_lock",
        "_shutdown_lock",
        "is_running",
        "listen",
        "port",
        "unix",
    )

    def __init__(
        self,
        listen: str,
        port: int,
        webhook_app: "WebhookAppClass",
        ssl_ctx: SSLContext | None,
        unix: str | Path | socket | None = None,
    ):
        if unix and not UNIX_AVAILABLE:
            raise RuntimeError("This OS does not support binding unix sockets.")
        self._http_server = HTTPServer(webhook_app, ssl_options=ssl_ctx)
        self.listen = listen
        self.port = port
        self.is_running = False
        self.unix = None
        if unix and isinstance(unix, socket):
            self.unix = unix
        elif unix:
            self.unix = bind_unix_socket(str(unix))
        self._server_lock = asyncio.Lock()
        self._shutdown_lock = asyncio.Lock()

    async def serve_forever(self, ready: asyncio.Event | None = None) -> None:
        async with self._server_lock:
            if self.unix:
                self._http_server.add_socket(self.unix)
            else:
                self._http_server.listen(self.port, address=self.listen)

            self.is_running = True
            if ready is not None:
                ready.set()

            _LOGGER.debug("Webhook Server started.")

    async def shutdown(self) -> None:
        async with self._shutdown_lock:
            if not self.is_running:
                _LOGGER.debug("Webhook Server is already shut down. Returning")
                return
            self.is_running = False
            self._http_server.stop()
            await self._http_server.close_all_connections()
            _LOGGER.debug("Webhook Server stopped")


class WebhookAppClass(tornado.web.Application):
    """Application used in the Webserver"""

    def __init__(
        self,
        webhook_path: str,
        bot: "Bot",
        update_queue: asyncio.Queue,
        secret_token: str | None = None,
    ):
        self.shared_objects = {
            "bot": bot,
            "update_queue": update_queue,
            "secret_token": secret_token,
        }
        handlers = [(rf"{webhook_path}/?", TelegramHandler, self.shared_objects)]
        tornado.web.Application.__init__(self, handlers)  # type: ignore

    def log_request(self, handler: tornado.web.RequestHandler) -> None:
        """Overrides the default implementation since we have our own logging setup."""


# pylint: disable=abstract-method
class TelegramHandler(tornado.web.RequestHandler):
    """BaseHandler that processes incoming requests from Telegram"""

    __slots__ = ("bot", "secret_token", "update_queue")

    SUPPORTED_METHODS = ("POST",)  # type: ignore[assignment]

    def initialize(self, bot: "Bot", update_queue: asyncio.Queue, secret_token: str) -> None:
        """Initialize for each request - that's the interface provided by tornado"""
        # pylint: disable=attribute-defined-outside-init
        self.bot = bot
        self.update_queue = update_queue
        self.secret_token = secret_token
        if secret_token:
            _LOGGER.debug(
                "The webhook server has a secret token, expecting it in incoming requests now"
            )

    def set_default_headers(self) -> None:
        """Sets default headers"""
        self.set_header("Content-Type", 'application/json; charset="utf-8"')

    async def post(self) -> None:
        """Handle incoming POST request"""
        _LOGGER.debug("Webhook triggered")
        self._validate_post()

        json_string = self.request.body.decode()
        data = json.loads(json_string)
        self.set_status(HTTPStatus.OK)
        _LOGGER.debug("Webhook received data: %s", json_string)

        try:
            update = Update.de_json(data, self.bot)
        except Exception as exc:
            _LOGGER.critical(
                "Something went wrong processing the data received from Telegram. "
                "Received data was *not* processed! Received data was: %r",
                data,
                exc_info=exc,
            )
            raise tornado.web.HTTPError(
                HTTPStatus.BAD_REQUEST, reason="Update could not be processed"
            ) from exc

        if update:
            _LOGGER.debug(
                "Received Update with ID %d on Webhook",
                # For some reason pylint thinks update is a general TelegramObject
                update.update_id,  # pylint: disable=no-member
            )

            # handle arbitrary callback data, if necessary
            if isinstance(self.bot, ExtBot):
                self.bot.insert_callback_data(update)

            await self.update_queue.put(update)

    def _validate_post(self) -> None:
        """Only accept requests with content type JSON"""
        ct_header = self.request.headers.get("Content-Type", None)
        if ct_header != "application/json":
            raise tornado.web.HTTPError(HTTPStatus.FORBIDDEN)
        # verifying that the secret token is the one the user set when the user set one
        if self.secret_token is not None:
            token = self.request.headers.get("X-Telegram-Bot-Api-Secret-Token")
            if not token:
                _LOGGER.debug("Request did not include the secret token")
                raise tornado.web.HTTPError(
                    HTTPStatus.FORBIDDEN, reason="Request did not include the secret token"
                )
            if token != self.secret_token:
                _LOGGER.debug("Request had the wrong secret token: %s", token)
                raise tornado.web.HTTPError(
                    HTTPStatus.FORBIDDEN, reason="Request had the wrong secret token"
                )

    def log_exception(
        self,
        typ: type[BaseException] | None,
        value: BaseException | None,
        tb: TracebackType | None,
    ) -> None:
        """Override the default logging and instead use our custom logging."""
        _LOGGER.debug(
            "%s - %s",
            self.request.remote_ip,
            "Exception in TelegramHandler",
            exc_info=(typ, value, tb) if typ and value and tb else value,
        )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultCachedMpeg4Gif,
    InlineQueryResultCachedVoice,
    InputTextMessageContent,
    MessageEntity,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def inline_query_result_cached_mpeg4_gif():
    return InlineQueryResultCachedMpeg4Gif(
        InlineQueryResultCachedMpeg4GifTestBase.id_,
        InlineQueryResultCachedMpeg4GifTestBase.mpeg4_file_id,
        title=InlineQueryResultCachedMpeg4GifTestBase.title,
        caption=InlineQueryResultCachedMpeg4GifTestBase.caption,
        parse_mode=InlineQueryResultCachedMpeg4GifTestBase.parse_mode,
        caption_entities=InlineQueryResultCachedMpeg4GifTestBase.caption_entities,
        input_message_content=InlineQueryResultCachedMpeg4GifTestBase.input_message_content,
        reply_markup=InlineQueryResultCachedMpeg4GifTestBase.reply_markup,
        show_caption_above_media=InlineQueryResultCachedMpeg4GifTestBase.show_caption_above_media,
    )


class InlineQueryResultCachedMpeg4GifTestBase:
    id_ = "id"
    type_ = "mpeg4_gif"
    mpeg4_file_id = "mpeg4 file id"
    title = "title"
    caption = "caption"
    parse_mode = "Markdown"
    caption_entities = [MessageEntity(MessageEntity.ITALIC, 0, 7)]
    input_message_content = InputTextMessageContent("input_message_content")
    reply_markup = InlineKeyboardMarkup([[InlineKeyboardButton("reply_markup")]])
    show_caption_above_media = True


class TestInlineQueryResultCachedMpeg4GifWithoutRequest(InlineQueryResultCachedMpeg4GifTestBase):
    def test_slot_behaviour(self, inline_query_result_cached_mpeg4_gif):
        inst = inline_query_result_cached_mpeg4_gif
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, inline_query_result_cached_mpeg4_gif):
        assert inline_query_result_cached_mpeg4_gif.type == self.type_
        assert inline_query_result_cached_mpeg4_gif.id == self.id_
        assert inline_query_result_cached_mpeg4_gif.mpeg4_file_id == self.mpeg4_file_id
        assert inline_query_result_cached_mpeg4_gif.title == self.title
        assert inline_query_result_cached_mpeg4_gif.caption == self.caption
        assert inline_query_result_cached_mpeg4_gif.parse_mode == self.parse_mode
        assert inline_query_result_cached_mpeg4_gif.caption_entities == tuple(
            self.caption_entities
        )
        assert (
            inline_query_result_cached_mpeg4_gif.input_message_content.to_dict()
            == self.input_message_content.to_dict()
        )
        assert (
            inline_query_result_cached_mpeg4_gif.reply_markup.to_dict()
            == self.reply_markup.to_dict()
        )
        assert (
            inline_query_result_cached_mpeg4_gif.show_caption_above_media
            == self.show_caption_above_media
        )

    def test_caption_entities_always_tuple(self):
        result = InlineQueryResultCachedMpeg4Gif(self.id_, self.mpeg4_file_id)
        assert result.caption_entities == ()

    def test_to_dict(self, inline_query_result_cached_mpeg4_gif):
        inline_query_result_cached_mpeg4_gif_dict = inline_query_result_cached_mpeg4_gif.to_dict()

        assert isinstance(inline_query_result_cached_mpeg4_gif_dict, dict)
        assert (
            inline_query_result_cached_mpeg4_gif_dict["type"]
            == inline_query_result_cached_mpeg4_gif.type
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["id"]
            == inline_query_result_cached_mpeg4_gif.id
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["mpeg4_file_id"]
            == inline_query_result_cached_mpeg4_gif.mpeg4_file_id
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["title"]
            == inline_query_result_cached_mpeg4_gif.title
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["caption"]
            == inline_query_result_cached_mpeg4_gif.caption
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["parse_mode"]
            == inline_query_result_cached_mpeg4_gif.parse_mode
        )
        assert inline_query_result_cached_mpeg4_gif_dict["caption_entities"] == [
            ce.to_dict() for ce in inline_query_result_cached_mpeg4_gif.caption_entities
        ]
        assert (
            inline_query_result_cached_mpeg4_gif_dict["input_message_content"]
            == inline_query_result_cached_mpeg4_gif.input_message_content.to_dict()
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["reply_markup"]
            == inline_query_result_cached_mpeg4_gif.reply_markup.to_dict()
        )
        assert (
            inline_query_result_cached_mpeg4_gif_dict["show_caption_above_media"]
            == inline_query_result_cached_mpeg4_gif.show_caption_above_media
        )

    def test_equality(self):
        a = InlineQueryResultCachedMpeg4Gif(self.id_, self.mpeg4_file_id)
        b = InlineQueryResultCachedMpeg4Gif(self.id_, self.mpeg4_file_id)
        c = InlineQueryResultCachedMpeg4Gif(self.id_, "")
        d = InlineQueryResultCachedMpeg4Gif("", self.mpeg4_file_id)
        e = InlineQueryResultCachedVoice(self.id_, "", "")

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Bot, ShippingAddress, ShippingQuery, Update, User
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def shipping_query(bot):
    sq = ShippingQuery(
        ShippingQueryTestBase.id_,
        ShippingQueryTestBase.from_user,
        ShippingQueryTestBase.invoice_payload,
        ShippingQueryTestBase.shipping_address,
    )
    sq.set_bot(bot)
    return sq


class ShippingQueryTestBase:
    id_ = "5"
    invoice_payload = "invoice_payload"
    from_user = User(0, "", False)
    shipping_address = ShippingAddress("GB", "", "London", "12 Grimmauld Place", "", "WC1")


class TestShippingQueryWithoutRequest(ShippingQueryTestBase):
    def test_slot_behaviour(self, shipping_query):
        inst = shipping_query
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id_,
            "invoice_payload": self.invoice_payload,
            "from": self.from_user.to_dict(),
            "shipping_address": self.shipping_address.to_dict(),
        }
        shipping_query = ShippingQuery.de_json(json_dict, offline_bot)
        assert shipping_query.api_kwargs == {}

        assert shipping_query.id == self.id_
        assert shipping_query.invoice_payload == self.invoice_payload
        assert shipping_query.from_user == self.from_user
        assert shipping_query.shipping_address == self.shipping_address
        assert shipping_query.get_bot() is offline_bot

    def test_to_dict(self, shipping_query):
        shipping_query_dict = shipping_query.to_dict()

        assert isinstance(shipping_query_dict, dict)
        assert shipping_query_dict["id"] == shipping_query.id
        assert shipping_query_dict["invoice_payload"] == shipping_query.invoice_payload
        assert shipping_query_dict["from"] == shipping_query.from_user.to_dict()
        assert shipping_query_dict["shipping_address"] == shipping_query.shipping_address.to_dict()

    def test_equality(self):
        a = ShippingQuery(self.id_, self.from_user, self.invoice_payload, self.shipping_address)
        b = ShippingQuery(self.id_, self.from_user, self.invoice_payload, self.shipping_address)
        c = ShippingQuery(self.id_, None, "", None)
        d = ShippingQuery(0, self.from_user, self.invoice_payload, self.shipping_address)
        e = Update(self.id_)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_answer(self, monkeypatch, shipping_query):
        async def make_assertion(*_, **kwargs):
            return kwargs["shipping_query_id"] == shipping_query.id

        assert check_shortcut_signature(
            ShippingQuery.answer, Bot.answer_shipping_query, ["shipping_query_id"], []
        )
        assert await check_shortcut_call(
            shipping_query.answer, shipping_query._bot, "answer_shipping_query"
        )
        assert await check_defaults_handling(shipping_query.answer, shipping_query._bot)

        monkeypatch.setattr(shipping_query._bot, "answer_shipping_query", make_assertion)
        assert await shipping_query.answer(ok=True)



#!/usr/bin/env python
# pylint: disable=redefined-builtin
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/]
"""This module contains classes related to gifs sent by bots."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._files.sticker import Sticker
from telegram._messageentity import MessageEntity
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional, de_list_optional, parse_sequence_arg
from telegram._utils.entities import parse_message_entities, parse_message_entity
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class GiftBackground(TelegramObject):
    """This object describes the background of a gift.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal if their :attr:`center_color`, :attr:`edge_color` and :attr:`text_color` are
    equal.

    .. versionadded:: 22.6

    Args:
        center_color (:obj:`int`): Center color of the background in RGB format.
        edge_color (:obj:`int`): Edge color of the background in RGB format.
        text_color (:obj:`int`): Text color of the background in RGB format.

    Attributes:
        center_color (:obj:`int`): Center color of the background in RGB format.
        edge_color (:obj:`int`): Edge color of the background in RGB format.
        text_color (:obj:`int`): Text color of the background in RGB format.

    """

    __slots__ = (
        "center_color",
        "edge_color",
        "text_color",
    )

    def __init__(
        self,
        center_color: int,
        edge_color: int,
        text_color: int,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.center_color: int = center_color
        self.edge_color: int = edge_color
        self.text_color: int = text_color

        self._id_attrs = (
            self.center_color,
            self.edge_color,
            self.text_color,
        )

        self._freeze()


class Gift(TelegramObject):
    """This object represents a gift that can be sent by the bot.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal if their :attr:`id` is equal.

    .. versionadded:: 21.8

    Args:
        id (:obj:`str`): Unique identifier of the gift.
        sticker (:class:`~telegram.Sticker`): The sticker that represents the gift.
        star_count (:obj:`int`): The number of Telegram Stars that must be paid to send the
            sticker.
        total_count (:obj:`int`, optional): The total number of the gifts of this type that can be
            sent by all users; for limited gifts only.
        remaining_count (:obj:`int`, optional): The number of remaining gifts of this type that can
            be sent by all users; for limited gifts only.
        upgrade_star_count (:obj:`int`, optional): The number of Telegram Stars that must be paid
            to upgrade the gift to a unique one.

            .. versionadded:: 21.10
        publisher_chat (:class:`telegram.Chat`, optional): Information about the chat that
            published the gift.

            .. versionadded:: 22.4
        personal_total_count (:obj:`int`, optional): The total number of gifts of this type that
            can be sent by the bot; for limited gifts only.

            .. versionadded:: 22.6
        personal_remaining_count (:obj:`int`, optional): The number of remaining gifts of this type
            that can be sent by the bot; for limited gifts only.

            .. versionadded:: 22.6
        background (:class:`GiftBackground`, optional): Background of the gift.

            .. versionadded:: 22.6
        is_premium (:obj:`bool`, optional): :obj:`True`, if the gift can only be purchased by
            Telegram Premium subscribers.

            .. versionadded:: 22.6
        has_colors (:obj:`bool`, optional): :obj:`True`, if the gift can be used (after being
            upgraded) to customize a user's appearance.

            .. versionadded:: 22.6
        unique_gift_variant_count (:obj:`int`, optional): The total number of different unique
            gifts that can be obtained by upgrading the gift.

            .. versionadded:: 22.6

    Attributes:
        id (:obj:`str`): Unique identifier of the gift.
        sticker (:class:`~telegram.Sticker`): The sticker that represents the gift.
        star_count (:obj:`int`): The number of Telegram Stars that must be paid to send the
            sticker.
        total_count (:obj:`int`): Optional. The total number of the gifts of this type that can be
            sent by all users; for limited gifts only.
        remaining_count (:obj:`int`): Optional. The number of remaining gifts of this type that can
            be sent by all users; for limited gifts only.
        upgrade_star_count (:obj:`int`): Optional. The number of Telegram Stars that must be paid
            to upgrade the gift to a unique one.

            .. versionadded:: 21.10
        publisher_chat (:class:`telegram.Chat`): Optional. Information about the chat that
            published the gift.

            .. versionadded:: 22.4
        personal_total_count (:obj:`int`): Optional. The total number of gifts of this type that
            can be sent by the bot; for limited gifts only.

            .. versionadded:: 22.6
        personal_remaining_count (:obj:`int`): Optional. The number of remaining gifts of this type
            that can be sent by the bot; for limited gifts only.

            .. versionadded:: 22.6
        background (:class:`GiftBackground`): Optional. Background of the gift.

            .. versionadded:: 22.6
        is_premium (:obj:`bool`): Optional. :obj:`True`, if the gift can only be purchased by
            Telegram Premium subscribers.

            .. versionadded:: 22.6
        has_colors (:obj:`bool`): Optional. :obj:`True`, if the gift can be used (after being
            upgraded) to customize a user's appearance.

            .. versionadded:: 22.6
        unique_gift_variant_count (:obj:`int`): Optional. The total number of different unique
            gifts that can be obtained by upgrading the gift.

            .. versionadded:: 22.6

    """

    __slots__ = (
        "background",
        "has_colors",
        "id",
        "is_premium",
        "personal_remaining_count",
        "personal_total_count",
        "publisher_chat",
        "remaining_count",
        "star_count",
        "sticker",
        "total_count",
        "unique_gift_variant_count",
        "upgrade_star_count",
    )

    def __init__(
        self,
        id: str,
        sticker: Sticker,
        star_count: int,
        total_count: int | None = None,
        remaining_count: int | None = None,
        upgrade_star_count: int | None = None,
        publisher_chat: Chat | None = None,
        personal_total_count: int | None = None,
        personal_remaining_count: int | None = None,
        background: GiftBackground | None = None,
        is_premium: bool | None = None,
        has_colors: bool | None = None,
        unique_gift_variant_count: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.id: str = id
        self.sticker: Sticker = sticker
        self.star_count: int = star_count
        self.total_count: int | None = total_count
        self.remaining_count: int | None = remaining_count
        self.upgrade_star_count: int | None = upgrade_star_count
        self.publisher_chat: Chat | None = publisher_chat
        self.personal_total_count: int | None = personal_total_count
        self.personal_remaining_count: int | None = personal_remaining_count
        self.background: GiftBackground | None = background
        self.is_premium: bool | None = is_premium
        self.has_colors: bool | None = has_colors
        self.unique_gift_variant_count: int | None = unique_gift_variant_count

        self._id_attrs = (self.id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "Gift":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["sticker"] = de_json_optional(data.get("sticker"), Sticker, bot)
        data["publisher_chat"] = de_json_optional(data.get("publisher_chat"), Chat, bot)
        data["background"] = de_json_optional(data.get("background"), GiftBackground, bot)
        return super().de_json(data=data, bot=bot)


class Gifts(TelegramObject):
    """This object represent a list of gifts.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal if their :attr:`gifts` are equal.

    .. versionadded:: 21.8

    Args:
        gifts (Sequence[:class:`Gift`]): The sequence of gifts.

    Attributes:
        gifts (tuple[:class:`Gift`]): The sequence of gifts.

    """

    __slots__ = ("gifts",)

    def __init__(
        self,
        gifts: Sequence[Gift],
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.gifts: tuple[Gift, ...] = parse_sequence_arg(gifts)

        self._id_attrs = (self.gifts,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "Gifts":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["gifts"] = de_list_optional(data.get("gifts"), Gift, bot)
        return super().de_json(data=data, bot=bot)


class GiftInfo(TelegramObject):
    """Describes a service message about a regular gift that was sent or received.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal if their :attr:`gift` is equal.

    .. versionadded:: 22.1

    Args:
        gift (:class:`Gift`): Information about the gift.
        owned_gift_id (:obj:`str`, optional): Unique identifier of the received gift for the bot;
            only present for gifts received on behalf of business accounts.
        convert_star_count (:obj:`int`, optional) Number of Telegram Stars that can be claimed by
            the receiver by converting the gift; omitted if conversion to Telegram Stars
            is impossible.
        prepaid_upgrade_star_count (:obj:`int`, optional): Number of Telegram Stars that were
            prepaid for the ability to upgrade the gift.
        can_be_upgraded (:obj:`bool`, optional): :obj:`True`, if the gift can be upgraded
            to a unique gift.
        text (:obj:`str`, optional): Text of the message that was added to the gift.
        entities (Sequence[:class:`telegram.MessageEntity`], optional): Special entities that
            appear in the text.
        is_private (:obj:`bool`, optional): :obj:`True`, if the sender and gift text are
            shown only to the gift receiver; otherwise, everyone will be able to see them.
        is_upgrade_separate (:obj:`bool`, optional): :obj:`True`, if the gift's upgrade was
            purchased after the gift was sent.

            .. versionadded:: 22.6
        unique_gift_number (:obj:`int`, optional): Unique number reserved for this gift when
            upgraded. See the number field in :class:`~telegram.UniqueGift`.

            .. versionadded:: 22.6

    Attributes:
        gift (:class:`Gift`): Information about the gift.
        owned_gift_id (:obj:`str`): Optional. Unique identifier of the received gift for the bot;
            only present for gifts received on behalf of business accounts.
        convert_star_count (:obj:`int`): Optional. Number of Telegram Stars that can be claimed by
            the receiver by converting the gift; omitted if conversion to Telegram Stars
            is impossible.
        prepaid_upgrade_star_count (:obj:`int`): Optional. Number of Telegram Stars that were
            prepaid for the ability to upgrade the gift.
        can_be_upgraded (:obj:`bool`): Optional. :obj:`True`, if the gift can be upgraded
            to a unique gift.
        text (:obj:`str`): Optional. Text of the message that was added to the gift.
        entities (Sequence[:class:`telegram.MessageEntity`]): Optional. Special entities that
            appear in the text.
        is_private (:obj:`bool`): Optional. :obj:`True`, if the sender and gift text are
            shown only to the gift receiver; otherwise, everyone will be able to see them.
        is_upgrade_separate (:obj:`bool`): Optional. :obj:`True`, if the gift's upgrade was
            purchased after the gift was sent.

            .. versionadded:: 22.6
        unique_gift_number (:obj:`int`): Optional. Unique number reserved for this gift when
            upgraded. See the number field in :class:`~telegram.UniqueGift`.

            .. versionadded:: 22.6

    """

    __slots__ = (
        "can_be_upgraded",
        "convert_star_count",
        "entities",
        "gift",
        "is_private",
        "is_upgrade_separate",
        "owned_gift_id",
        "prepaid_upgrade_star_count",
        "text",
        "unique_gift_number",
    )

    def __init__(
        self,
        gift: Gift,
        owned_gift_id: str | None = None,
        convert_star_count: int | None = None,
        prepaid_upgrade_star_count: int | None = None,
        can_be_upgraded: bool | None = None,
        text: str | None = None,
        entities: Sequence[MessageEntity] | None = None,
        is_private: bool | None = None,
        unique_gift_number: int | None = None,
        is_upgrade_separate: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.gift: Gift = gift
        # Optional
        self.owned_gift_id: str | None = owned_gift_id
        self.convert_star_count: int | None = convert_star_count
        self.prepaid_upgrade_star_count: int | None = prepaid_upgrade_star_count
        self.can_be_upgraded: bool | None = can_be_upgraded
        self.text: str | None = text
        self.entities: tuple[MessageEntity, ...] = parse_sequence_arg(entities)
        self.is_private: bool | None = is_private
        self.unique_gift_number: int | None = unique_gift_number
        self.is_upgrade_separate: bool | None = is_upgrade_separate

        self._id_attrs = (self.gift,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "GiftInfo":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["gift"] = de_json_optional(data.get("gift"), Gift, bot)
        data["entities"] = de_list_optional(data.get("entities"), MessageEntity, bot)

        return super().de_json(data=data, bot=bot)

    def parse_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`text`
        from a given :class:`telegram.MessageEntity` of :attr:`entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`entities`.

        Returns:
            :obj:`str`: The text of the given entity.

        Raises:
            RuntimeError: If the gift info has no text.

        """
        if not self.text:
            raise RuntimeError("This GiftInfo has no 'text'.")

        return parse_message_entity(self.text, entity)

    def parse_entities(self, types: list[str] | None = None) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this gift info's text filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.

        Raises:
            RuntimeError: If the gift info has no text.

        """
        if not self.text:
            raise RuntimeError("This GiftInfo has no 'text'.")

        return parse_message_entities(self.text, self.entities, types)


class AcceptedGiftTypes(TelegramObject):
    """This object describes the types of gifts that can be gifted to a user or a chat.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal if their :attr:`unlimited_gifts`, :attr:`limited_gifts`,
    :attr:`unique_gifts`, :attr:`premium_subscription` and :attr:`gifts_from_channels` are equal.

    .. versionadded:: 22.1
    .. versionchanged:: 22.6
        :attr:`gifts_from_channels` is now considered for equality checks.

    Args:
        unlimited_gifts (:class:`bool`): :obj:`True`, if unlimited regular gifts are accepted.
        limited_gifts (:class:`bool`): :obj:`True`, if limited regular gifts are accepted.
        unique_gifts (:class:`bool`): :obj:`True`, if unique gifts or gifts that can be upgraded
            to unique for free are accepted.
        premium_subscription (:class:`bool`): :obj:`True`, if a Telegram Premium subscription
            is accepted.
        gifts_from_channels (:obj:`bool`): :obj:`True`, if transfers of unique gifts from channels
            are accepted

            .. versionadded:: 22.6

    Attributes:
        unlimited_gifts (:class:`bool`): :obj:`True`, if unlimited regular gifts are accepted.
        limited_gifts (:class:`bool`): :obj:`True`, if limited regular gifts are accepted.
        unique_gifts (:class:`bool`): :obj:`True`, if unique gifts or gifts that can be upgraded
            to unique for free are accepted.
        premium_subscription (:class:`bool`): :obj:`True`, if a Telegram Premium subscription
            is accepted.
        gifts_from_channels (:obj:`bool`): :obj:`True`, if transfers of unique gifts from channels
            are accepted

            .. versionadded:: 22.6

    """

    __slots__ = (
        "gifts_from_channels",
        "limited_gifts",
        "premium_subscription",
        "unique_gifts",
        "unlimited_gifts",
    )

    def __init__(
        self,
        unlimited_gifts: bool,
        limited_gifts: bool,
        unique_gifts: bool,
        premium_subscription: bool,
        gifts_from_channels: bool,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.unlimited_gifts: bool = unlimited_gifts
        self.limited_gifts: bool = limited_gifts
        self.unique_gifts: bool = unique_gifts
        self.premium_subscription: bool = premium_subscription
        self.gifts_from_channels: bool = gifts_from_channels

        self._id_attrs = (
            self.unlimited_gifts,
            self.limited_gifts,
            self.unique_gifts,
            self.premium_subscription,
            self.gifts_from_channels,
        )

        self._freeze()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
"""This module contains the classes for Telegram Stars affiliates."""

from typing import TYPE_CHECKING

from telegram._chat import Chat
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class AffiliateInfo(TelegramObject):
    """Contains information about the affiliate that received a commission via this transaction.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`affiliate_user`, :attr:`affiliate_chat`,
    :attr:`commission_per_mille`, :attr:`amount`, and :attr:`nanostar_amount` are equal.

    .. versionadded:: 21.9

    Args:
        affiliate_user (:class:`telegram.User`, optional): The bot or the user that received an
            affiliate commission if it was received by a bot or a user
        affiliate_chat (:class:`telegram.Chat`, optional): The chat that received an affiliate
            commission if it was received by a chat
        commission_per_mille (:obj:`int`): The number of Telegram Stars received by the affiliate
            for each 1000 Telegram Stars received by the bot from referred users
        amount (:obj:`int`): Integer amount of Telegram Stars received by the affiliate from the
            transaction, rounded to 0; can be negative for refunds
        nanostar_amount (:obj:`int`, optional): The number of
            :tg-const:`~telegram.constants.Nanostar.VALUE` shares of Telegram
            Stars received by the affiliate; from
            :tg-const:`~telegram.constants.NanostarLimit.MIN_AMOUNT` to
            :tg-const:`~telegram.constants.NanostarLimit.MAX_AMOUNT`;
            can be negative for refunds

    Attributes:
        affiliate_user (:class:`telegram.User`): Optional. The bot or the user that received an
            affiliate commission if it was received by a bot or a user
        affiliate_chat (:class:`telegram.Chat`): Optional. The chat that received an affiliate
            commission if it was received by a chat
        commission_per_mille (:obj:`int`): The number of Telegram Stars received by the affiliate
            for each 1000 Telegram Stars received by the bot from referred users
        amount (:obj:`int`): Integer amount of Telegram Stars received by the affiliate from the
            transaction, rounded to 0; can be negative for refunds
        nanostar_amount (:obj:`int`): Optional. The number of
            :tg-const:`~telegram.constants.Nanostar.VALUE` shares of Telegram
            Stars received by the affiliate; from
            :tg-const:`~telegram.constants.NanostarLimit.MIN_AMOUNT` to
            :tg-const:`~telegram.constants.NanostarLimit.MAX_AMOUNT`;
            can be negative for refunds
    """

    __slots__ = (
        "affiliate_chat",
        "affiliate_user",
        "amount",
        "commission_per_mille",
        "nanostar_amount",
    )

    def __init__(
        self,
        commission_per_mille: int,
        amount: int,
        affiliate_user: "User | None" = None,
        affiliate_chat: "Chat | None" = None,
        nanostar_amount: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ) -> None:
        super().__init__(api_kwargs=api_kwargs)
        self.affiliate_user: User | None = affiliate_user
        self.affiliate_chat: Chat | None = affiliate_chat
        self.commission_per_mille: int = commission_per_mille
        self.amount: int = amount
        self.nanostar_amount: int | None = nanostar_amount

        self._id_attrs = (
            self.affiliate_user,
            self.affiliate_chat,
            self.commission_per_mille,
            self.amount,
            self.nanostar_amount,
        )
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "AffiliateInfo":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["affiliate_user"] = de_json_optional(data.get("affiliate_user"), User, bot)
        data["affiliate_chat"] = de_json_optional(data.get("affiliate_chat"), Chat, bot)

        return super().de_json(data=data, bot=bot)
